@@ -2,14 +2,13 @@
 
 use std::ops::{Deref, DerefMut};
 
-use serde::{de::Visitor, Deserialize, Deserializer, Serialize, Serializer};
-
-pub const NAME: &str = "$__v8_magic_bytestring";
-pub const FIELD_PTR: &str = "$__v8_magic_bytestring_ptr";
-pub const FIELD_LEN: &str = "$__v8_magic_bytestring_len";
+use super::transl8::{FromV8, ToV8};
+use crate::magic::transl8::impl_magic;
+use crate::Error;
 
 #[derive(PartialEq, Eq, Clone, Debug)]
 pub struct ByteString(pub Vec<u8>);
+impl_magic!(ByteString);
 
 impl ByteString {
   pub fn new() -> ByteString {
@@ -81,45 +80,43 @@ impl AsMut<[u8]> for ByteString {
   }
 }
 
-impl Serialize for ByteString {
-  fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
-  where
-    S: Serializer,
-  {
-    use serde::ser::SerializeStruct;
-
-    let mut s = serializer.serialize_struct(NAME, 1)?;
-    s.serialize_field(FIELD_PTR, &(self.0.as_ptr() as usize))?;
-    s.serialize_field(FIELD_LEN, &self.0.len())?;
-    s.end()
+impl ToV8 for ByteString {
+  fn to_v8<'a>(
+    &self,
+    scope: &mut v8::HandleScope<'a>,
+  ) -> Result<v8::Local<'a, v8::Value>, crate::Error> {
+    let v =
+      v8::String::new_from_one_byte(scope, self, v8::NewStringType::Normal)
+        .unwrap();
+    Ok(v.into())
   }
 }
 
-impl<'de> Deserialize<'de> for ByteString {
-  fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
-  where
-    D: Deserializer<'de>,
-  {
-    struct ValueVisitor {}
-
-    impl<'de> Visitor<'de> for ValueVisitor {
-      type Value = ByteString;
-
-      fn expecting(
-        &self,
-        formatter: &mut std::fmt::Formatter,
-      ) -> std::fmt::Result {
-        formatter.write_str("a serde_v8::ByteString")
-      }
-
-      fn visit_byte_buf<E>(self, v: Vec<u8>) -> Result<Self::Value, E>
-      where
-        E: serde::de::Error,
-      {
-        Ok(ByteString(v))
-      }
+impl FromV8 for ByteString {
+  fn from_v8(
+    scope: &mut v8::HandleScope,
+    value: v8::Local<v8::Value>,
+  ) -> Result<Self, crate::Error> {
+    let v8str = v8::Local::<v8::String>::try_from(value)
+      .map_err(|_| Error::ExpectedString)?;
+    if !v8str.contains_only_onebyte() {
+      return Err(Error::ExpectedLatin1);
     }
-
-    deserializer.deserialize_struct(NAME, &[], ValueVisitor {})
+    let len = v8str.length();
+    let mut buffer = Vec::with_capacity(len);
+    // SAFETY: we set length == capacity (see previous line),
+    // before immediately writing into that buffer and sanity check with an assert
+    #[allow(clippy::uninit_vec)]
+    unsafe {
+      buffer.set_len(len);
+      let written = v8str.write_one_byte(
+        scope,
+        &mut buffer,
+        0,
+        v8::WriteOptions::NO_NULL_TERMINATION,
+      );
+      assert!(written == len);
+    }
+    Ok(ByteString(buffer))
   }
 }
